## R Version 3.4.2
## This code was adapted from:
## Gompert, Z., Lucas, L. K., Buerkle, C. A., Forister, M. L., Fordyce, J. A., & Nice, C. C. (2014). Admixture
##     and the organization of genetic diversity in a butterfly species complex revealed through common and
##     rare genetic variants. Molecular Ecology, 23(18), 4555-4573.

## Here we:
## 1. Create a genetic distance matrix from the .geno file (with all 9s changed to "NA").
## 2. Read in a geographic distance matrix.
## 3. Create a taxon distance matrix (same species = 0, different species = 1).
## 4. Run a mixed model regression MCMC function.

## variable definitions
## Y = genetic distance matrix (dist)
## X = predictor distance matrixes (list of dist)
## n = number of populations or matrix rows (integer)
## p = number of predictor matrixes
## nmcmc = number of mcmc iterations (integer)
## burnin = number of initial mcmc iterations to discard (integer)


## file processing for geno file. Change 9 to NA.
## cat erisor.geno | perl -p -i -e 's// /g' | perl -p -i -e 's/9/NA/g' > erisor_NA.geno

library(adegenet)

L<-923
N<-155

## Read in genotypes
gen<-matrix(scan("erisor_NA.geno",n=N*L,sep=" "),nrow=L,ncol=N,byrow=FALSE)

## create genetic distance matrix
G<-matrix(NA,nrow=N,ncol=N)
for(i in 1:(N-1)){
  for(j in (i+1):N){
    G[i,j]<-mean(abs(gen[,i]-gen[,j]),na.rm=TRUE)/2
    G[j,i]<-G[i,j]
  }
}  

geo<-read.table("geo_dist_km_index_indivs.csv",sep=" ",header=TRUE)
geo<-as.matrix(geo[,-1])

spec<-read.table("indivs_species.csv",sep=",",header=FALSE)

## Create matrix of species distances: 0 for same and 1 for different
S<-matrix(NA,nrow=N,ncol=N)
for(i in 1:(N-1)){
  for(j in (i+1):N){
    if(spec[i,2]==spec[j,2]){
      S[i,j]<-0
      S[j,i]<-0
    }  
    else{
      S[i,j]<-1
      S[j,i]<-1
    }  
  }
}  


mmrMcmc<-function(Y=NA,X1=NULL,X2=NULL,npop=NA,nmcmc=5000,burnin=1000,thin=5,nchain=2){
  
  library(rjags)
  load.module("dic")
  
  n<-(npop*(npop-1))/2
  Y<-log(Y/(1-Y))
  Y<-Y-mean(Y)
  
  if (is.null(X1)==FALSE){
    X1<-X1-mean(X1)
  }
  if (is.null(X2)==FALSE){    
    X2<-X2-mean(X2) 
  }
  ## create random effect
  R1<-numeric(n)
  R2<-numeric(n)
  k<-1
  for(i in 1:(npop-1)){
    for(j in (i+1):npop){
      R1[k]<-i
      R2[k]<-j
      k<-k+1
    }
  }
  
  ## specifies the linear model for jags, single path or differences
  jagsModel="
  model {
  
  ## normal likelihood and linear model: mu = beta0 + beta X + alpha_i + alpha_j
  for (i in 1:n) {
  Y[i] ~ dnorm(mu[i],taue)
  mu[i]<-beta0 + inprod(X[i,],beta) + alpha[pop1[i]] + alpha[pop2[i]]
  }
  
  ## covariate priors
  for (k in 1:p) {
  beta[k] ~ dnorm(0,0.001)    
  }
  ## intercept prior
  beta0 ~ dnorm(0,0.001)
  ## population effect priors
  for (j in 1:npop){
  alpha[j] ~ dnorm(0, taua)
  }    
  
  ## residual priors
  taua ~ dgamma(1,0.01)
  taue ~ dgamma(1,0.01)
  }  
  "
  
  ## format data
  if (is.null(X1)){
    X<-as.matrix(as.vector(X2))
    p<-1
  }
  else if (is.null(X2)){
    X<-as.matrix(as.vector(X1))
    p<-1
  }
  else{
    X<-as.matrix(cbind(as.vector(X1),as.vector(X2)))
    p<-2
  }        
  data<-list(Y=as.vector(Y),X=X,n=n,p=p,npop=npop,pop1=R1,pop2=R2)
  
  ## model initiatlization
  model<-jags.model(textConnection(jagsModel),data=data,n.chains=nchain,n.adapt=0) ## add back init
  update(model,n.iter=burnin)
  
  ## mcmc estimation of model parameters
  #mcmcSamples<-coda.samples(model=model,variable.names=c("beta0","beta","tau","rho"), n.iter=nmcmc,thin=thin)## the code below allows dic and paramters to be sampled together
  mcmcSamples<-coda.samples(model=model,variable.names=c("beta0","beta","taue","taua"), n.iter=nmcmc,thin=thin)
  dicSams<-dic.samples(model=model,n.iter=nmcmc,thin=thin,type="pD")
  
  
  return(list(param=mcmcSamples,dic=dicSams))
  
}
Gd<-as.dist(G)
X1<-as.dist(geo) 
X2<-as.dist(S)

str(Gd)
str(X1)
str(X2)

outCommonFull<-mmrMcmc(Y=as.vector(Gd),X1=as.vector(X1),X2=as.vector(X2),npop=N,nmcmc=10000,burnin=2000,thin=5,nchain=3)
outCommonDistOnly<-mmrMcmc(Y=as.vector(Gd),X1=as.vector(X1),X2=NULL,npop=66,nmcmc=10000,burnin=2000,thin=5,nchain=3)


